# -*- coding: utf-8 -*-

import sys, json
from os.path import dirname, abspath

project_path = dirname(dirname(abspath(__file__)))
projec_full_path = project_path + r'\py_protoc'
sys.path.append(projec_full_path)

from py_protoc import msg_stock_pb2 as msg_stock
from py_protoc import msg_stock_pb2 as msg_stock
from py_protoc import msg_stock_common_pb2 as msg_stock_common


class KaisaProtoc(object):

    def __init__(self):
        pass

    @staticmethod
    def get_ping_bstring(reqId=1):

        pb_reqhead = msg_stock_common.ReqHead()
        pb_reqhead.reqId = str(reqId)
        pb_reqhead.reqType = 3
        pb_string = pb_reqhead.SerializeToString()
        return pb_string

    @staticmethod
    def get_subscribe_bstring(symbollist, reqId=1):

        pb_rule = msg_stock.Rule()
        pb_stockcode = msg_stock.StockCode()
        for secucode_dict in symbollist:
            pb_stockcode_one = pb_rule.stockCodes.add()
            pb_stockcode_one.code = secucode_dict['code']
            pb_stockcode_one.market = 2
            pb_stockcode_one.language = 0

        pb_rule.types.append(1)
        pb_rule.types.append(2)

        pb_reqhead = msg_stock_common.ReqHead()
        pb_reqhead.reqId = str(reqId)
        pb_reqhead.reqType = 1
        pb_reqhead.data = pb_rule.SerializeToString()

        pb_string = pb_reqhead.SerializeToString()
        return pb_string

    @staticmethod
    def get_unsubscribe_bstring(symbollist, reqId=1):

        pb_rule = msg_stock.Rule()
        pb_stockcode = msg_stock.StockCode()
        for secucode_dict in symbollist:
            pb_stockcode_one = pb_rule.stockCodes.add()
            pb_stockcode_one.code = secucode_dict['code']
            pb_stockcode_one.market = 2
            pb_stockcode_one.language = 0

        pb_rule.types.append(1)
        pb_rule.types.append(2)

        pb_reqhead = msg_stock_common.ReqHead()
        pb_reqhead.reqId = str(reqId)
        pb_reqhead.reqType = 2
        pb_reqhead.data = pb_rule.SerializeToString()

        pb_string = pb_reqhead.SerializeToString()
        return pb_string


    @staticmethod
    def rec_bstring_to_json(bstring):

        pb_reshead = msg_stock_common.ResHead()
        pb_reshead.ParseFromString(bstring)

        if (pb_reshead.msg=='SUCCESS'):
            pass
        else:
            print("error.")
            print(pb_reshead.msg)
            return None

        if (pb_reshead.type==1):
            quote = msg_stock.Quote()
            quote.ParseFromString(pb_reshead.data)
            body_json = KaisaProtoc.quote_bstring_to_json(quote)
            data_type = 'quote'

        elif (pb_reshead.type==2):
            ticklist = msg_stock.TickList()
            ticklist.ParseFromString(pb_reshead.data)
            body_json = KaisaProtoc.ticklist_bstring_to_json(ticklist)
            data_type = 'tick'

        return {'type': data_type,
                'data': body_json}

    @staticmethod
    def quote_bstring_to_json(quote):

        return {'market':quote.market,
                'code':quote.code,
                'now':quote.now,
                'high': quote.high,
                'low': quote.low,
                'volume': quote.volume,
                'amount': quote.amount,
                'quote_time': quote.quote_time.seconds+quote.quote_time.nanos/1000000000.0
                }

    @staticmethod
    def ticklist_bstring_to_json(ticklist):

        return {'market': ticklist.market,
                'code': ticklist.code,
                'tick': [KaisaProtoc.tick_bstring_to_json(item) for item in ticklist.tick]
                }

    @staticmethod
    def tick_bstring_to_json(tick):

        return {'now': tick.now,
                'cur_volume': tick.cur_volume,
                'tick_flag': tick.tick_flag,
                'tick_vi': tick.tick_vi,
                'tick_time': tick.tick_time.seconds+tick.tick_time.nanos/1000000000.0
                }

if __name__=='__main__':

    print('start.')





# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# ===================================================================

"""
This code provide basic market and trade function for KaisaGlobal-Open.
Pls. note that only HK market is supported now.

developed by KaisaGlobal quant team.
2020.12.11
"""


import os, sys, json

import numpy as np
import pandas as pd

import traceback
import requests
#from ws4py.client.threadedclient import WebSocketClient
import threading

import websocket
from copy import deepcopy
import time
import sys, json


import sys, json
from os.path import dirname, abspath


project_path = dirname(dirname(abspath(__file__)))
projec_full_path = project_path + r'\common'
sys.path.append(projec_full_path)


from com import *
from KaisaCrypto import *
from utils import *
from KaisaProtoc import *

kcrypto = KaisaCrypto()
kprotoc = KaisaProtoc()




class KaisaGateway(object):

    req_id = 0

    # gateway - market, trade;
    gateway_auth_status = False

    # ws
    market_ws_alive = False
    trade_ws_alive = False

    # trade;
    trade_connect_status = False
    trade_auth_status = False

    # token
    gatewayToken = None
    tradeToken = None

    _market_ws = None
    _trade_ws = None

    _market_connect_status = False

    _ws_trade_connect_status = False

    _market_connect_msg = ""


    def __init__(self):
        pass

    def jq_user_config(self, user_id, user_pwd, account_code, account_pwd, openapi_token, ip_address=None, environment='paper'):

        self.user_id = user_id
        self.user_pwd = user_pwd
        self.account_code = account_code
        self.account_pwd = account_pwd
        self.openapi_token = openapi_token
        self.environment = environment
        self.root_url = KAISA_ROOT_URL_SIM if self.environment in ["simulate", "paper"] else KAISA_ROOT_URL
        self._update_urls()

        self.m_req_id = 0
        self.trade_req_id = 0
        self.ip_address = ip_address


    def _update_urls(self):

        update_url = lambda url: url.replace("__kaisarooturl__", self.root_url)

        self.CRYPTO_RSA_URL = update_url(CRYPTO_RSA_URL)
        self.AUTHENTICA_URL = update_url(AUTHENTICA_URL)
        self.QUOTE_URL = update_url(QUOTE_URL)

        self.REST_DATA_HOST = update_url(REST_DATA_HOST)
        self.WEBSOCKET_DATA_HOST = update_url(WEBSOCKET_DATA_HOST)

        self.ClientByMobile_URL = update_url(ClientByMobile_URL)
        self.REST_HOST = update_url(REST_HOST)
        self.WEBSOCKET_TRADE_HOST = update_url(WEBSOCKET_TRADE_HOST)

        self.TRADE_CRYPTO_RSA_URL = update_url(TRADE_CRYPTO_RSA_URL)

    def write_error(self, data):
        print("error: {}".format(data))

    def write_log(self, data):
        print("log: {}".format(data))

    def _authentica(self, auth_username: str, auth_password: str) -> None:
        """
        获取网关登录令牌;
        :param auth_username:
        :param auth_password:
        :return:
        """

        if self.gateway_auth_status:
            return self.gateway_auth_status, self.gateway_auth_msg

        # only for token of gateway.
        timestamp_ = generate_timestamp()
        sign_ = kcrypto.encrypt_md5("username{}Timestamp{}".format(auth_username, timestamp_))

        auth_username_encrypt = kcrypto.encrypt_rsa_username(auth_username, crypto_rsa_url=self.CRYPTO_RSA_URL)
        auth_password_encrypt = kcrypto.encrypt_aes_password(auth_password, "MAKRET")

        params = {
            "username": auth_username_encrypt,
            "password": auth_password_encrypt,
            "grant_type": "password",
            "scope": openapi_scope
        }

        ipAddress = self.ip_address
        if ipAddress:
            params["ipAddress"] = ipAddress

        headerParam = {"srvType":"open","ipAddress":ipAddress,"phoneModel":"","deviceId":"","deviceToken":"","macAddress":"","channelId":"","country":"","sysVersion":"","apiVersion":"","platform":""}
        headerParam = json.dumps(headerParam)

        authorization_str = "basic {}".format(self.openapi_token)
        headers = {
            "Authorization": authorization_str,
            "Content-Type": "application/x-www-form-urlencoded",
            "Sign": sign_,
            "Timestamp": timestamp_,
            "headerParam": headerParam
        }
        response = requests.post(
            url=self.AUTHENTICA_URL, params=params, headers=headers
        )

        self.gateway_auth_msg = ""

        data = response.json()
        if response.status_code // 100 == 2:
            self.write_log("网关认证成功")
            if data['success']:
                self.write_log("获取登录令牌成功")
                token_body = data["body"]["accessToken"]
                self.gatewayToken = f"bearer {token_body}"
                self.token = self.gatewayToken
                self.gateway_auth_status = True
            else:
                self.token = None
                self.gatewayToken = None
                self.write_log("获取登录令牌失败")
                self.write_error(data)
        else:
            self.write_log("网关认证失败")
            self.write_error(data)

        return self.gateway_auth_status, self.gateway_auth_msg

    def gateway_auth(self):
        return self._authentica(self.user_id, self.user_pwd)

    def market_connect_ws(self):
        ret_ok, ret_msg = self.gateway_auth()
        if ret_ok:
            self.market_connect_ws0()
        time.sleep(5)

    def do_market_requests(self, url, reqdata):

        reqdata = self._market_req_decorate(reqdata)
        method = reqdata["method"] if ("method" in reqdata) else "POST"
        headers = reqdata["headers"] if ("headers" in reqdata) else None
        params = reqdata["params"] if ("params" in reqdata) else None
        data = reqdata["data"] if ("data" in reqdata) else None

        response = requests.request(
            method,
            url,
            headers=headers,
            params=params,
            data=json.dumps(data)
        )
        status_code = response.status_code
        resp_data = response.json()
        return status_code, resp_data


    def do_trade_requests(self, url, reqdata, reqtype="normal"):

        reqdata = self._trade_req_decorate(reqdata, reqtype)
        method = reqdata["method"] if ("method" in reqdata) else "POST"
        headers = reqdata["headers"] if ("headers" in reqdata) else None
        params = reqdata["params"] if ("params" in reqdata) else None
        data = reqdata["data"] if ("data" in reqdata) else None

        response = requests.request(
            method,
            url,
            headers=headers,
            params=params,
            data=data
        )
        status_code = response.status_code
        resp_data = response.json()
        return status_code, resp_data


    def _market_req_decorate(self, reqdata):

        reqdata['headers'] = {
            "Authorization": self.gatewayToken,
            "Content-Type": "application/json"
        }
        return reqdata

    def query_all_symbollist(self):

        contractdata_all = []
        beginpos = 0
        count = 1000
        while True:
            symbols = self.query_symbollist(beginpos=beginpos, count=count)
            if symbols is None:
                break
            contractdata_all += symbols
            if len(symbols)<count:
                break
            beginpos += count
        if len(contractdata_all)>0:
            self.write_log("合约信息查询成功")
        else:
            self.write_log("合约信息查询失败")

        return contractdata_all


    def query_symbollist(self, beginpos=0, count=1000):

        self.req_id += 1
        data = {
            "reqtype": QUERY_CONTRACT,
            "reqid": self.req_id,
            "session": "",
            "data": {
                "marketid": HKSE_MARKET,
                "idtype": 1,
                "beginpos": beginpos,
                "count": count,
                "getquote": 1
            }
        }

        reqdata = {"data": data}
        status_code, resp_data = self.do_market_requests(url=self.QUOTE_URL, reqdata=reqdata)
        if status_code//100==2:
            symbols = resp_data['data']['symbol']
        else:
            symbols = None
        return symbols


    def query_history_kline(self, symbol='01638', market='HK', klinetype='1d', adjfactortype='0', pagenum=1, pagesize=100):
        """
        query history kline (only for hk market)
        """
        post_url = MARKET_POST_URL+"/quoteInfo/kLine"

        klinetype2klineid = {'1min': 1,
                             '5min': 3,
                             '15min': 5,
                             '30min': 6,
                             '1hour': 7,
                             '1d': 10
                             }
        klineid = klinetype2klineid.get(klinetype)
        rqdata = {
            'code': symbol,
            'market': market,
            'quoteTypeEnumCode': klineid,
            'adjFactorType': adjfactortype,
            'pageNum': pagenum,
            'pageSize': pagesize
        }
        response = requests.request(
            'POST',
            post_url,
            headers={'Content-Type':'application/json'},
            params=None,
            data=json.dumps(rqdata)
        )
        status_code = response.status_code
        resp_data = response.json()
        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']['records']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def query_history_tick(self, symbol='00700', market='HK', pagesize=10, pagenum=1):

        post_url = MARKET_POST_URL + "/stock/quote/tick"
        rqdata = {
            'code': symbol,
            'market': market,
            'size': str(pagesize),
            'index': str(pagesize*pagenum)
        }
        response = requests.request(
            'POST',
            post_url,
            headers={'Content-Type':'application/json'},
            params=None,
            data=json.dumps(rqdata)
        )
        status_code = response.status_code
        resp_data = response.json()
        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

    ################################财务数据##################################
    ###### 基础财务数据(主要财务数据、三大报表)
    def _handle_req_ret(self, period, report_type, code, market):
        # 公共统一处理请求和返回数据

        body = {
            "reportType": report_type,
            "periodMark": period,
            "stockCode": code,
            "stockMarket": market
        }

        response = requests.request(
            method='POST',
            url=BASIC_FIN_POST_URL,
            headers={'Content-Type': 'application/json'},
            params=None,
            data=json.dumps(body),
        )
        resp_data = response.json()
        resp_status = resp_data['success']
        # if resp_status:
        #     body_data = resp_data['body']
        # else:
        #     body_data = {'retCode': resp_data['retCode'],
        #                  'retMsg': resp_data['retMsg']}
        if resp_status is False:
            return pd.DataFrame()
        try:
            _df_ = pd.DataFrame(resp_data['body']['data'])
        except:
            _df_ = pd.DataFrame()

        return _df_

    def get_basic_fin_data(self, code='00700', market='hkg', period=12, report_type='1'):
        '''
        periodMark	integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
        reportType	string 1：主要指标
        stockCode	string 股票代码
        stockMarket	string 港股：hkg，美股：usa,a股：chn
        Returns:
            df:
            主要财务指标  季度/年度的财务数据
        '''
        basic_fin_df = self._handle_req_ret(period, report_type, code, market)

        return basic_fin_df

    def get_income_fin_data(self, code='00700', market='hkg', period=12, report_type='2'):
        '''
        Args:
            code: string 股票代码
            market: string 港股：hkg，美股：usa,a股：chn
            period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
            report_type: string 2：利润表
        Returns:
            df:
            income 季度/年度的财务数据
        '''
        income_fin_df = self._handle_req_ret(period, report_type, code, market)

        return income_fin_df

    def get_balance_fin_data(self, code='00700', market='hkg', period=12, report_type='3'):
        '''
        Args:
            code: string 股票代码
            market: string 港股：hkg，美股：usa,a股：chn
            period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
            report_type: string 3：资产负债表
        Returns:
            df:
            balance 季度/年度的财务数据
        '''
        balance_fin_df = self._handle_req_ret(period, report_type, code, market)

        return balance_fin_df

    def get_cashflow_fin_data(self, code='00700', market='hkg', period=12, report_type='4'):
        '''
        Args:
            code: string 股票代码
            market: string 港股：hkg，美股：usa,a股：chn
            period: integer($int32) null-全部 3-一季度，6-半年度，9-三季度，12-年度
            report_type: string 4：现金流量表
        Returns:
            df
            cashflow  季度/年度的财务数据
        '''
        cashflow_fin_df = self._handle_req_ret(period, report_type, code, market)

        return cashflow_fin_df

    ###### 衍生财务数据(财务因子)

    ################################财务数据##################################

    def query_detailstate(self, symbol='01638', market='HK'):

        post_url = MARKET_POST_URL+ "/quoteInfo/todayDetailState"
        rqdata = {
            'code': symbol,
            'market': market,
            'hasQuoteInfo': True,
            'hasQuoteExt': True,
            'hasQuoteStat': True,
            'hasQuoteFinancial': True,
            'hasQuotePro': True
        }
        response = requests.request(
            'POST',
            post_url,
            headers={'Content-Type': 'application/json', 'headerParam':json.dumps({'apiVersion':"1.2.0"})},
            params=None,
            data=json.dumps(rqdata)
        )
        status_code = response.status_code
        resp_data = response.json()
        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
            del body_data['assetInfoResp']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def market_connect_ws0(self):

        threading.Thread(target=self.__market_connect_ws, daemon=True).start()

    def get_market_connect_ws_status(self):

        return self._market_connect_status, self._market_connect_msg

    def __market_connect_ws(self):

        if self._market_connect_status:
            return

        header = {"Authorization": self.gatewayToken,
                  "deviceId":"xxx",
                  "platform":"open",
                  "srvType":"open"}

        data = {"deviceId": "xxx",
                  "platform": "open",
                  "srvType": "open"}

        header = {"Authorization": self.gatewayToken,
                  "headerParam": json.dumps(data)
                    }
        # create_connection--
        '''
        ws = None
        try:
            ws = CG_Client('wss://i.cg.net/wi/ws')
            ws.connect()
            ws.run_forever()
        except KeyboardInterrupt:
            ws.close()
        '''
        #self._market_ws = WebSocketClient(MARKET_WS_URL, headers=header)
        try:
            self._market_ws.connect()
            self._market_ws.run_forever()
        except Exception as exp:
            print('exp:{}'.format(exp))

        self._market_ws = websocket.create_connection(MARKET_WS_URL, header=header)
        self._market_ping_th = threading.Thread(target=self._market_ping, args=(self._market_ws,), daemon=True)
        self._market_ping_th.start()

        self._market_connect_status = True
        self._market_connect_msg = ""

        print('wait to receive.')
        while True:

            text = self._market_ws.recv()
            data = kprotoc.rec_bstring_to_json(text)
            if data is None:
                continue
            self.on_market_data(data)


    def on_market_packet(self, packet):

        self.on_market_data(packet)

    def on_market_data(self, data):
        pass

    def subscribe_marketdata(self, code_list):
        """
        推送行情和分笔；
        """
        self.req_id += 1
        bstring = kprotoc.get_subscribe_bstring(code_list, self.req_id)
        self._market_ws.send_binary(bstring)

    def unsubscribe_marketdata(self, code_list):
        """
        取消推送行情和分笔
        :param symbols:
        :return:
        """
        self.req_id+=1
        bstring = kprotoc.get_unsubscribe_bstring(code_list, self.req_id)
        self._market_ws.send_binary(bstring)

    def market_heartbeat(self):

        self.req_id += 1
        ping_bstring = kprotoc.get_ping_bstring(self.req_id)
        self._market_ws.send_binary(ping_bstring)

    def send_market_packet(self, packet):

        text = json.dumps(packet)
        self._market_ws.send(text)

    def generate_req(self, reqtype: int, data: dict) -> dict:

        self.req_id += 1
        req = {
            "reqtype": reqtype,
            "reqid": self.req_id,
            "session": "",
            "data": data
        }
        return req


    def _trade_req_decorate(self, reqdata, reqtype="normal"):

        json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
        method = reqdata['method'] if 'method' in reqdata else "POST"
        headers = reqdata['headers'] if 'headers' in reqdata else {}
        data = reqdata['data'] if 'data' in reqdata else {}
        params = reqdata['params'] if 'params' in reqdata else {}

        this_timestamp = generate_timestamp()

        if (reqtype=="connect") and (method=="POST") and ('q' in reqdata['data']):
            headers = {"Content-Type": "application/json",
                       "Authorization": self.token
                        }
            params = deepcopy(reqdata['data'])
            reqdata = {'method':method,
                       'headers':headers,
                       'data':json_dumps(reqdata['data']),
                       'params': params
                       }
            return reqdata

        if (reqtype=="login"):

            headers['Content-Type'] = "application/json"
            headers["Authorization"] = self.token
            if reqtype=="login":
                data['Authorization'] = self.token
            if self.tradeToken is not None:
                headers['X-Trade-Token'] = self.tradeToken
            if reqtype=="normal":
                if 'tradeToken' in data:
                    del data['tradeToken']
            if method=="POST":
                header_ = reqdata['data'].copy()
                if self.tradeToken is not None:
                    header_["tradeToken"] = self.tradeToken
                headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
                headers["Timestamp"] = this_timestamp

            reqdata = {"method":method,
                       "data":json_dumps(data),
                       "headers":headers,
                       "params":json_dumps(params)}
            return reqdata

        headers = {"Content-Type": "application/json",
                   "Authorization": self.gatewayToken,
                   "X-Trade-Token": self.tradeToken
                   }

        if method=="GET":
            if 'params' in reqdata:
                header_ = reqdata['params'].copy()
            else:
                header_ = {}
            header_["Authorization"] = self.token
            headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="GET")
            headers["Timestamp"] = this_timestamp

        if method=="POST":
            header_ = data.copy()
            if self.tradeToken is not None:
                header_["tradeToken"] = self.tradeToken
            headers["Sign"] = self.__sign(header_=header_, timestamp_=this_timestamp, request_="POST")
            headers["Timestamp"] = this_timestamp

        if (method=="POST") and ('q' not in data):
            q_string = json_dumps(data)
            q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
            data = {'q': q_string}

        reqdata = {"method":method,
                   "data":json_dumps(data),
                   "headers":headers,
                   "params":params}

        return reqdata

    def _sign(self, reqdata, reqtype="normal"):

        json_dumps = lambda item: json.dumps(item, separators=(',', ':'))
        method = reqdata['method']
        headers = reqdata['headers'] if 'headers' in reqdata else {}
        data = reqdata['data'] if 'data' in reqdata else {}
        params = reqdata['params'] if 'params' in reqdata else {}

        if (reqtype=="connect") and (method=="POST") and ('q' in reqdata['data']):
            headers = {"Content-Type": "application/json",
                       "Authorization": self.token
                        }
            params = deepcopy(reqdata['data'])
            reqdata = {'method':method,
                       'headers':headers,
                       'data':json_dumps(reqdata['data']),
                       'params': params
                       }
            return reqdata

        if (reqtype=="login"):

            headers['Content-Type'] = "application/json"
            headers["Authorization"] = self.token
            if reqtype=="login":
                data['Authorization'] = self.token

            if self.tradeToken is not None:
                headers['X-Trade-Token'] = self.tradeToken

            if reqtype=="normal":
                if 'tradeToken' in data:
                    del data['tradeToken']

            if method=="POST":
                timestamp_ = generate_timestamp()
                header_ = reqdata['data'].copy()
                if self.tradeToken is not None:
                    header_["tradeToken"] = self.tradeToken
                headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
                headers["Timestamp"] = timestamp_

            reqdata = {"method":method,
                       "data":json_dumps(data),
                       "headers":headers,
                       "params":json_dumps(params)}

            return reqdata

        headers = {"Content-Type": "application/json",
                   "Authorization": self.gatewayToken,
                   "X-Trade-Token": self.tradeToken
                   }

        if method=="GET":
            header_ = reqdata['params'].copy()
            header_["Authorization"] = self.token
            headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="GET")
            headers["Timestamp"] = timestamp_

        if method=="POST":
            timestamp_ = generate_timestamp()
            header_ = data.copy()
            if self.tradeToken is not None:
                header_["tradeToken"] = self.tradeToken
            headers["Sign"] = self.__sign(header_=header_, timestamp_=timestamp_, request_="POST")
            headers["Timestamp"] = timestamp_

        if (method=="POST") and ('q' not in data):
            q_string = json_dumps(data)
            q_string = kcrypto.encrypt_aes_password_forQ(q_string, "SECRET")
            data = {'q': q_string}

        reqdata = {"method":method,
                   "data":json_dumps(data),
                   "headers":headers,
                   "params":params}

        return reqdata


    def __sign(self, header_: dict = {}, request_=None, timestamp_=None):

        sign_ = None
        str_ = ""
        for key_ in sorted(header_):
            str_ += str(key_)+str(header_[key_])
        str_ += "Timestamp"+timestamp_
        sign_ = kcrypto.encrypt_md5(str_)
        return sign_


    def trade_connect(self):
        """
        交易连接
        :return: trade_connect_status
        """
        if self.gateway_auth():
            en_password_trade = kcrypto.encrypt_aes_password(self.account_pwd, type="TRADE")
            self.en_password_trade = en_password_trade
            secret_q = kcrypto.encrypt_rsa_secretQ(crypto_rsa_url = self.TRADE_CRYPTO_RSA_URL)
            data = {
                "q": secret_q,
                "accountCode": self.account_code
            }
            url = self.REST_HOST + "/v1/account/shakeHand"
            reqdata = {"data": data}

            status_code, resp_data = self.do_trade_requests(url, reqdata, "connect")
            self.sessionId = resp_data['body']['sessionId']
            self.trade_connect_status = True

        self.trade_connect_msg = ''
        return self.trade_connect_status, self.trade_connect_msg


    def trade_auth(self):

        data = {
            "channelType": "INTERNET",
            "accountCode": self.account_code,
            "password": self.en_password_trade,
            "secondAuthFromOther": "Y",
            "sessionId": self.sessionId
        }
        reqdata = {"data": data}

        url = self.REST_HOST + "/v1/account/login"
        status_code, resp_data = self.do_trade_requests(url, reqdata, "login")

        if status_code//100==2:
            self.tradeToken = resp_data['body']['tradeToken']
            self.trade_auth_status = True
        else:
            self.tradeToken = None
            self.trade_auth_status = False
        self.trade_auth_msg = ''
        return self.trade_auth_status, self.trade_auth_msg


    def trade_connect_ws(self):
        threading.Thread(target=self._start_trade_ws, daemon=True).start()

    def get_trade_connect_ws_status(self):
        return self._ws_trade_connect_status, ""

    def _market_ping(self, this_ws):
        """
        :param this_ws:
        :return:
        """
        while True:
            self.market_heartbeat()
            time.sleep(5)

    def _trade_ping(self, this_ws):
        """
        :param this_ws:
        :return:
        """
        pass


    def _start_trade_ws(self):

        if self._ws_trade_connect_status:
            return True

        #
        data = {
            "channelType": "INTERNET",
            "accountCode": self.account_code,
            "password": self.en_password_trade,
            "ipAddress": "",
            "secondAuthFromOther": "Y",
            "sessionId": self.sessionId,
        }

        host = self.WEBSOCKET_TRADE_HOST
        header = {"Authorization": self.token}
        # # create_connection--
        self._trade_ws = websocket.create_connection(host, header=header)

        req = self.generate_req(LOGIN, data)

        req = json.dumps(req)
        self._trade_ws.send(req)
        self._trade_ping_th = threading.Thread(target=self._trade_ping, args=(self._trade_ws,), daemon=True)
        self._trade_ping_th.start()
        self._ws_trade_connect_status = True

        while True:
            text = self._trade_ws.recv()
            self.on_trade_packet(text)


    def on_trade_packet(self, packet):

        if len(packet)==0:
            return
        data = json.loads(packet)
        if data.get('reqtype',0)==2:
            req_data = {"ts":data['data']["ts"]}
            req_data = self.generate_req(PONG, req_data)
            self._trade_ws.send(json.dumps(req_data))
        else:
            data = json.loads(packet)
            self.on_trade_data(packet)


    def send_order(self, bsFlag='B', price=550, qty=600, code='00700'):

        return self.place_order(bsFlag=bsFlag, price=price, qty=qty, code=code)

    def place_order(self, bsFlag='B', price=550, qty=600, code='00700'):

        data = {'channelType': "I",
                'exchangeCode': 'HKEX',
                'accountCode': self.account_code,
                'productCode': code,
                'price': price, 'qty': qty,
                'bsFlag': bsFlag, 'orderType': 'L',
                'tradeToken': self.tradeToken}

        reqdata = {"data": data,
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/order/orders/place"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def cancel_order(self, sys_orderid):

        data = {
            "channelType": "I",
            "accountCode": self.account_code,
            "orderID": sys_orderid,
            "tradeToken": self.tradeToken,
        }
        reqdata = {"data": data,
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/order/orders/cancel"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def query_portfolio(self):

        reqdata = {"method":"GET",
                   "params": {'accountCode': self.account_code}}

        url = self.REST_HOST+"/v1/account/accounts/portfolio"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def query_balance(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/account/accounts/balance"
        status_code, resp_data = self.do_trade_requests(url, reqdata)
        resp_data = resp_data.json()
        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def query_position(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/account/accounts/position"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data


    def query_order(self):

        reqdata = {"method": "GET",
                   "params": {'accountCode': self.account_code}}
        url = self.REST_HOST + "/v1/order/orders"
        status_code, resp_data = self.do_trade_requests(url, reqdata)

        resp_status = resp_data['success']
        if resp_status:
            body_data = resp_data['body']
        else:
            body_data = {'retCode': resp_data['retCode'],
                         'retMsg': resp_data['retMsg']
                         }
        return resp_status, body_data

if __name__=="__main__":

    print("Hello, Kaisa.")




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg_stock.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()
import sys, json
from os.path import dirname, abspath

project_path = dirname(dirname(abspath(__file__)))
projec_full_path = project_path + r'\py_protoc'
sys.path.append(projec_full_path)

from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from py_protoc import msg_stock_common_pb2 as msg__stock__common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg_stock.proto',
  package='MsgStock',
  syntax='proto3',
  serialized_options=b'\n#com.kaisaglobal.proto.stock.message\200\001\000\210\001\001\220\001\000',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fmsg_stock.proto\x12\x08MsgStock\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x16msg_stock_common.proto\"p\n\x04Rule\x12\'\n\nstockCodes\x18\x01 \x03(\x0b\x32\x13.MsgStock.StockCode\x12/\n\x05types\x18\x02 \x03(\x0e\x32 .MsgStockCommon.SubscriptionType\x12\x0e\n\x06\x66ields\x18\x03 \x03(\t\"\x9e\x01\n\tStockCode\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12&\n\x06market\x18\x02 \x01(\x0e\x32\x16.MsgStockCommon.Market\x12*\n\x08language\x18\x03 \x01(\x0e\x32\x18.MsgStockCommon.Language\x12/\n\x05types\x18\x04 \x03(\x0e\x32 .MsgStockCommon.SubscriptionType\"\xfb\x02\n\x05Quote\x12\x0e\n\x06market\x18\x01 \x01(\r\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0b\n\x03now\x18\x03 \x01(\t\x12\x0c\n\x04open\x18\x04 \x01(\t\x12\x0c\n\x04high\x18\x05 \x01(\t\x12\x0b\n\x03low\x18\x06 \x01(\t\x12\x12\n\nlast_close\x18\x07 \x01(\t\x12\x0e\n\x06volume\x18\x08 \x01(\t\x12\x0e\n\x06\x61mount\x18\t \x01(\t\x12\x13\n\x0bupdate_flag\x18\n \x03(\x04\x12.\n\nquote_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\trecv_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\ngrey_quote\x18\r \x01(\x0b\x32\x13.MsgStock.GreyQuote\x12\x15\n\rquantityRatio\x18\x0e \x01(\t\x12\r\n\x05\x63lose\x18\x0f \x01(\t\x12\x0f\n\x07tick_vi\x18\x10 \x01(\t\x12\x16\n\x0etrading_status\x18\x11 \x01(\t\"d\n\tGreyQuote\x12\x17\n\x0f\x63ur_trade_count\x18\x01 \x01(\t\x12\x0f\n\x07lotSize\x18\x02 \x01(\t\x12\x16\n\x0eorderSellPrice\x18\x03 \x01(\t\x12\x15\n\rorderBuyPrice\x18\x04 \x01(\t\"F\n\x08TickList\x12\x0e\n\x06market\x18\x01 \x01(\r\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x1c\n\x04tick\x18\x03 \x03(\x0b\x32\x0e.MsgStock.Tick\"\xd1\x01\n\x04Tick\x12\x0e\n\x06req_id\x18\x01 \x01(\r\x12\x0b\n\x03now\x18\x02 \x01(\t\x12\x12\n\ncur_volume\x18\x03 \x01(\t\x12\x11\n\ttick_flag\x18\x04 \x01(\r\x12\x0f\n\x07tick_vi\x18\x05 \x01(\t\x12-\n\ttick_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\trecv_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0egrey_tick_time\x18\x08 \x01(\t\"\xac\x01\n\x0fOrderStallsList\x12\x0e\n\x06market\x18\x01 \x01(\r\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\torderSell\x18\x04 \x03(\x0b\x32\x15.MsgStock.OrderStalls\x12\'\n\x08orderBuy\x18\x05 \x03(\x0b\x32\x15.MsgStock.OrderStalls\"Q\n\x0bOrderStalls\x12\r\n\x05level\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t\x12\x0e\n\x06volume\x18\x03 \x01(\t\x12\x14\n\x0c\x62roker_count\x18\x04 \x01(\t\"\xce\x01\n\nBrokerList\x12\x0e\n\x06market\x18\x01 \x01(\r\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12$\n\nbrokerSell\x18\x04 \x03(\x0b\x32\x10.MsgStock.Broker\x12#\n\tbrokerBuy\x18\x05 \x03(\x0b\x32\x10.MsgStock.Broker\x12-\n\trecv_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"1\n\x06\x42roker\x12\x14\n\x0c\x62roker_level\x18\x01 \x01(\t\x12\x11\n\tbroker_id\x18\x02 \x03(\t*S\n\x07Tick_VI\x12\x10\n\x0cTick_VI_None\x10\x00\x12\x05\n\x01X\x10\x01\x12\x05\n\x01\x44\x10\x02\x12\x05\n\x01R\x10\x03\x12\x05\n\x01M\x10\x04\x12\x05\n\x01N\x10\x05\x12\x05\n\x01Y\x10\x06\x12\x05\n\x01P\x10\x07\x12\x05\n\x01U\x10\x08*\xe0\x01\n\nUpdateFlag\x12\x07\n\x03now\x10\x00\x12\x08\n\x04open\x10\x01\x12\x08\n\x04high\x10\x02\x12\x07\n\x03low\x10\x03\x12\n\n\x06volume\x10\x04\x12\n\n\x06\x61mount\x10\x05\x12\x0e\n\nlast_close\x10\x06\x12\x13\n\x0f\x63ur_trade_count\x10\x07\x12\x0b\n\x07lotSize\x10\x08\x12\x12\n\x0eorderSellPrice\x10\t\x12\x11\n\rorderBuyPrice\x10\n\x12\x11\n\rquantityRatio\x10\x0b\x12\t\n\x05\x63lose\x10\x0c\x12\n\n\x06tickVi\x10\r\x12\x11\n\rtradingStatus\x10\x0e\x42.\n#com.kaisaglobal.proto.stock.message\x80\x01\x00\x88\x01\x01\x90\x01\x00\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,msg__stock__common__pb2.DESCRIPTOR,])

_TICK_VI = _descriptor.EnumDescriptor(
  name='Tick_VI',
  full_name='MsgStock.Tick_VI',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Tick_VI_None', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='X', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='D', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='R', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='M', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='N', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Y', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='P', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='U', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1647,
  serialized_end=1730,
)
_sym_db.RegisterEnumDescriptor(_TICK_VI)

Tick_VI = enum_type_wrapper.EnumTypeWrapper(_TICK_VI)
_UPDATEFLAG = _descriptor.EnumDescriptor(
  name='UpdateFlag',
  full_name='MsgStock.UpdateFlag',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='now', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='open', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='high', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='low', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='volume', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='amount', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='last_close', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='cur_trade_count', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='lotSize', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='orderSellPrice', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='orderBuyPrice', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='quantityRatio', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='close', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='tickVi', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='tradingStatus', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1733,
  serialized_end=1957,
)
_sym_db.RegisterEnumDescriptor(_UPDATEFLAG)

UpdateFlag = enum_type_wrapper.EnumTypeWrapper(_UPDATEFLAG)
Tick_VI_None = 0
X = 1
D = 2
R = 3
M = 4
N = 5
Y = 6
P = 7
U = 8
now = 0
open = 1
high = 2
low = 3
volume = 4
amount = 5
last_close = 6
cur_trade_count = 7
lotSize = 8
orderSellPrice = 9
orderBuyPrice = 10
quantityRatio = 11
close = 12
tickVi = 13
tradingStatus = 14



_RULE = _descriptor.Descriptor(
  name='Rule',
  full_name='MsgStock.Rule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='stockCodes', full_name='MsgStock.Rule.stockCodes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='types', full_name='MsgStock.Rule.types', index=1,
      number=2, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fields', full_name='MsgStock.Rule.fields', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=86,
  serialized_end=198,
)


_STOCKCODE = _descriptor.Descriptor(
  name='StockCode',
  full_name='MsgStock.StockCode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStock.StockCode.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStock.StockCode.market', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='language', full_name='MsgStock.StockCode.language', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='types', full_name='MsgStock.StockCode.types', index=3,
      number=4, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=201,
  serialized_end=359,
)


_QUOTE = _descriptor.Descriptor(
  name='Quote',
  full_name='MsgStock.Quote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStock.Quote.market', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStock.Quote.code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='now', full_name='MsgStock.Quote.now', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='open', full_name='MsgStock.Quote.open', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high', full_name='MsgStock.Quote.high', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='low', full_name='MsgStock.Quote.low', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_close', full_name='MsgStock.Quote.last_close', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='volume', full_name='MsgStock.Quote.volume', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='MsgStock.Quote.amount', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update_flag', full_name='MsgStock.Quote.update_flag', index=9,
      number=10, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quote_time', full_name='MsgStock.Quote.quote_time', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recv_time', full_name='MsgStock.Quote.recv_time', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grey_quote', full_name='MsgStock.Quote.grey_quote', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quantityRatio', full_name='MsgStock.Quote.quantityRatio', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='close', full_name='MsgStock.Quote.close', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_vi', full_name='MsgStock.Quote.tick_vi', index=15,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trading_status', full_name='MsgStock.Quote.trading_status', index=16,
      number=17, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=362,
  serialized_end=741,
)


_GREYQUOTE = _descriptor.Descriptor(
  name='GreyQuote',
  full_name='MsgStock.GreyQuote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cur_trade_count', full_name='MsgStock.GreyQuote.cur_trade_count', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lotSize', full_name='MsgStock.GreyQuote.lotSize', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orderSellPrice', full_name='MsgStock.GreyQuote.orderSellPrice', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orderBuyPrice', full_name='MsgStock.GreyQuote.orderBuyPrice', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=743,
  serialized_end=843,
)


_TICKLIST = _descriptor.Descriptor(
  name='TickList',
  full_name='MsgStock.TickList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStock.TickList.market', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStock.TickList.code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick', full_name='MsgStock.TickList.tick', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=845,
  serialized_end=915,
)


_TICK = _descriptor.Descriptor(
  name='Tick',
  full_name='MsgStock.Tick',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='req_id', full_name='MsgStock.Tick.req_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='now', full_name='MsgStock.Tick.now', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cur_volume', full_name='MsgStock.Tick.cur_volume', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_flag', full_name='MsgStock.Tick.tick_flag', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_vi', full_name='MsgStock.Tick.tick_vi', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_time', full_name='MsgStock.Tick.tick_time', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recv_time', full_name='MsgStock.Tick.recv_time', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grey_tick_time', full_name='MsgStock.Tick.grey_tick_time', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=918,
  serialized_end=1127,
)


_ORDERSTALLSLIST = _descriptor.Descriptor(
  name='OrderStallsList',
  full_name='MsgStock.OrderStallsList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStock.OrderStallsList.market', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStock.OrderStallsList.code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStock.OrderStallsList.time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orderSell', full_name='MsgStock.OrderStallsList.orderSell', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orderBuy', full_name='MsgStock.OrderStallsList.orderBuy', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1130,
  serialized_end=1302,
)


_ORDERSTALLS = _descriptor.Descriptor(
  name='OrderStalls',
  full_name='MsgStock.OrderStalls',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='MsgStock.OrderStalls.level', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='MsgStock.OrderStalls.price', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='volume', full_name='MsgStock.OrderStalls.volume', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='broker_count', full_name='MsgStock.OrderStalls.broker_count', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1304,
  serialized_end=1385,
)


_BROKERLIST = _descriptor.Descriptor(
  name='BrokerList',
  full_name='MsgStock.BrokerList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStock.BrokerList.market', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStock.BrokerList.code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStock.BrokerList.time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='brokerSell', full_name='MsgStock.BrokerList.brokerSell', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='brokerBuy', full_name='MsgStock.BrokerList.brokerBuy', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='recv_time', full_name='MsgStock.BrokerList.recv_time', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1388,
  serialized_end=1594,
)


_BROKER = _descriptor.Descriptor(
  name='Broker',
  full_name='MsgStock.Broker',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='broker_level', full_name='MsgStock.Broker.broker_level', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='broker_id', full_name='MsgStock.Broker.broker_id', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1596,
  serialized_end=1645,
)

_RULE.fields_by_name['stockCodes'].message_type = _STOCKCODE
_RULE.fields_by_name['types'].enum_type = msg__stock__common__pb2._SUBSCRIPTIONTYPE
_STOCKCODE.fields_by_name['market'].enum_type = msg__stock__common__pb2._MARKET
_STOCKCODE.fields_by_name['language'].enum_type = msg__stock__common__pb2._LANGUAGE
_STOCKCODE.fields_by_name['types'].enum_type = msg__stock__common__pb2._SUBSCRIPTIONTYPE
_QUOTE.fields_by_name['quote_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_QUOTE.fields_by_name['recv_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_QUOTE.fields_by_name['grey_quote'].message_type = _GREYQUOTE
_TICKLIST.fields_by_name['tick'].message_type = _TICK
_TICK.fields_by_name['tick_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TICK.fields_by_name['recv_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ORDERSTALLSLIST.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ORDERSTALLSLIST.fields_by_name['orderSell'].message_type = _ORDERSTALLS
_ORDERSTALLSLIST.fields_by_name['orderBuy'].message_type = _ORDERSTALLS
_BROKERLIST.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_BROKERLIST.fields_by_name['brokerSell'].message_type = _BROKER
_BROKERLIST.fields_by_name['brokerBuy'].message_type = _BROKER
_BROKERLIST.fields_by_name['recv_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['Rule'] = _RULE
DESCRIPTOR.message_types_by_name['StockCode'] = _STOCKCODE
DESCRIPTOR.message_types_by_name['Quote'] = _QUOTE
DESCRIPTOR.message_types_by_name['GreyQuote'] = _GREYQUOTE
DESCRIPTOR.message_types_by_name['TickList'] = _TICKLIST
DESCRIPTOR.message_types_by_name['Tick'] = _TICK
DESCRIPTOR.message_types_by_name['OrderStallsList'] = _ORDERSTALLSLIST
DESCRIPTOR.message_types_by_name['OrderStalls'] = _ORDERSTALLS
DESCRIPTOR.message_types_by_name['BrokerList'] = _BROKERLIST
DESCRIPTOR.message_types_by_name['Broker'] = _BROKER
DESCRIPTOR.enum_types_by_name['Tick_VI'] = _TICK_VI
DESCRIPTOR.enum_types_by_name['UpdateFlag'] = _UPDATEFLAG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Rule = _reflection.GeneratedProtocolMessageType('Rule', (_message.Message,), {
  'DESCRIPTOR' : _RULE,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.Rule)
  })
_sym_db.RegisterMessage(Rule)

StockCode = _reflection.GeneratedProtocolMessageType('StockCode', (_message.Message,), {
  'DESCRIPTOR' : _STOCKCODE,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.StockCode)
  })
_sym_db.RegisterMessage(StockCode)

Quote = _reflection.GeneratedProtocolMessageType('Quote', (_message.Message,), {
  'DESCRIPTOR' : _QUOTE,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.Quote)
  })
_sym_db.RegisterMessage(Quote)

GreyQuote = _reflection.GeneratedProtocolMessageType('GreyQuote', (_message.Message,), {
  'DESCRIPTOR' : _GREYQUOTE,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.GreyQuote)
  })
_sym_db.RegisterMessage(GreyQuote)

TickList = _reflection.GeneratedProtocolMessageType('TickList', (_message.Message,), {
  'DESCRIPTOR' : _TICKLIST,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.TickList)
  })
_sym_db.RegisterMessage(TickList)

Tick = _reflection.GeneratedProtocolMessageType('Tick', (_message.Message,), {
  'DESCRIPTOR' : _TICK,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.Tick)
  })
_sym_db.RegisterMessage(Tick)

OrderStallsList = _reflection.GeneratedProtocolMessageType('OrderStallsList', (_message.Message,), {
  'DESCRIPTOR' : _ORDERSTALLSLIST,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.OrderStallsList)
  })
_sym_db.RegisterMessage(OrderStallsList)

OrderStalls = _reflection.GeneratedProtocolMessageType('OrderStalls', (_message.Message,), {
  'DESCRIPTOR' : _ORDERSTALLS,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.OrderStalls)
  })
_sym_db.RegisterMessage(OrderStalls)

BrokerList = _reflection.GeneratedProtocolMessageType('BrokerList', (_message.Message,), {
  'DESCRIPTOR' : _BROKERLIST,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.BrokerList)
  })
_sym_db.RegisterMessage(BrokerList)

Broker = _reflection.GeneratedProtocolMessageType('Broker', (_message.Message,), {
  'DESCRIPTOR' : _BROKER,
  '__module__' : 'msg_stock_pb2'
  # @@protoc_insertion_point(class_scope:MsgStock.Broker)
  })
_sym_db.RegisterMessage(Broker)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg_stock_dz.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg_stock_dz.proto',
  package='MsgStockDz',
  syntax='proto3',
  serialized_options=b'\n#com.kaisaglobal.proto.stock.message\200\001\001\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12msg_stock_dz.proto\x12\nMsgStockDz\x1a\x1fgoogle/protobuf/timestamp.proto\"2\n\x04Head\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0e\n\x06packid\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"\x16\n\x05Login\x12\r\n\x05token\x18\x01 \x01(\t\"&\n\x08\x45rrorMsg\x12\r\n\x05\x65rror\x18\x01 \x01(\r\x12\x0b\n\x03msg\x18\x02 \x01(\t\"&\n\x07KickMsg\x12\x0e\n\x06reason\x18\x01 \x01(\r\x12\x0b\n\x03msg\x18\x02 \x01(\t\"I\n\x03\x42id\x12\r\n\x05level\x18\x01 \x01(\r\x12\r\n\x05price\x18\x02 \x01(\x01\x12\x0e\n\x06volume\x18\x03 \x01(\r\x12\x14\n\x0c\x62roker_count\x18\x04 \x01(\r\"\xc4\x04\n\x05Quote\x12\x13\n\x0bupdate_flag\x18\x01 \x03(\x04\x12\x0e\n\x06market\x18\x02 \x01(\r\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12(\n\x04time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\ttick_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\tpack_type\x18\x06 \x01(\r\x12\x12\n\nquote_type\x18\x07 \x01(\r\x12\x11\n\ttick_flag\x18\x08 \x01(\r\x12\x11\n\ttick_type\x18\t \x01(\r\x12\x0b\n\x03now\x18\n \x01(\x01\x12\x0c\n\x04open\x18\x0b \x01(\x01\x12\x0c\n\x04high\x18\x0c \x01(\x01\x12\x0b\n\x03low\x18\r \x01(\x01\x12\x0b\n\x03\x61vg\x18\x0e \x01(\x01\x12\x0e\n\x06settle\x18\x0f \x01(\x01\x12\x0c\n\x04hold\x18\x10 \x01(\r\x12\x0e\n\x06volume\x18\x11 \x01(\x04\x12\x0e\n\x06\x61mount\x18\x12 \x01(\x01\x12\x12\n\nlast_close\x18\x13 \x01(\x01\x12\x13\n\x0blast_settle\x18\x14 \x01(\x01\x12\x11\n\tlast_hold\x18\x15 \x01(\x01\x12\x17\n\x0f\x63ur_trade_count\x18\x16 \x01(\r\x12\x13\n\x0btrade_count\x18\x17 \x01(\r\x12\x12\n\ncur_volume\x18\x18 \x01(\r\x12\x12\n\nrise_limit\x18\x19 \x01(\x01\x12\x12\n\nfall_limit\x18\x1a \x01(\x01\x12\x1d\n\x04sell\x18\x1b \x03(\x0b\x32\x0f.MsgStockDz.Bid\x12\x1c\n\x03\x62uy\x18\x1c \x03(\x0b\x32\x0f.MsgStockDz.Bid\"0\n\x06\x42roker\x12\x14\n\x0c\x62roker_level\x18\x01 \x01(\r\x12\x10\n\x08\x62rokerid\x18\x02 \x03(\r\"\xa5\x01\n\nBrokerList\x12\x0e\n\x06market\x18\x01 \x01(\r\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x07 \x01(\r\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12 \n\x04sell\x18\x04 \x03(\x0b\x32\x12.MsgStockDz.Broker\x12\x1f\n\x03\x62uy\x18\x05 \x03(\x0b\x32\x12.MsgStockDz.Broker\"l\n\x06SymExt\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0e\n\x06market\x18\x02 \x01(\r\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12(\n\x04time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"a\n\tMarketExt\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0e\n\x06market\x18\x02 \x01(\r\x12(\n\x04time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\x0c\"P\n\x08OtherExt\x12\x0c\n\x04type\x18\x01 \x01(\r\x12(\n\x04time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"9\n\x0bUpdateEvent\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x0e\n\x06market\x18\x02 \x01(\r\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t*\xef\x01\n\nUpdateFlag\x12\x07\n\x03now\x10\x00\x12\x08\n\x04open\x10\x01\x12\x08\n\x04high\x10\x02\x12\x07\n\x03low\x10\x03\x12\x07\n\x03\x61vg\x10\x04\x12\n\n\x06settle\x10\x05\x12\x08\n\x04hold\x10\x06\x12\n\n\x06volume\x10\x07\x12\n\n\x06\x61mount\x10\x08\x12\x0e\n\nlast_close\x10\t\x12\x0f\n\x0blast_settle\x10\n\x12\r\n\tlast_hold\x10\x0b\x12\x13\n\x0f\x63ur_trade_count\x10\x0c\x12\x0f\n\x0btrade_count\x10\r\x12\x0e\n\ncur_volume\x10\x0e\x12\x0e\n\nrise_limit\x10\x0f\x12\x0e\n\nfall_limit\x10\x10*\xba\x03\n\tReqTypeNo\x12\x12\n\x0eReqTypeNo_NULL\x10\x00\x12\x10\n\x0cHeart_TypeNo\x10\x01\x12\x16\n\x12Market_Push_TypeNo\x10\x02\x12\x17\n\x13\x42roker_Queue_TypeNo\x10\x03\x12!\n\x1dProduct_Extension_Data_TypeNo\x10\x04\x12 \n\x1cMarket_Expansion_Data_TypeNo\x10\x05\x12 \n\x1cOther_Data_Extensions_TypeNo\x10\x06\x12#\n\x1f\x44\x61ta_Update_Notification_TypeNo\x10\n\x12)\n%Product_Extension_Notification_TypeNo\x10\x0b\x12(\n$Market_Expansion_Notification_TypeNo\x10\x0c\x12-\n)Other_Data_Extensions_Notification_TypeNo\x10\r\x12\x1a\n\x16Login_Agreement_TypeNo\x10\x14\x12\x17\n\x13\x45rror_Packet_TypeNo\x10\x15\x12\x11\n\rKicked_TypeNo\x10\x16\x42.\n#com.kaisaglobal.proto.stock.message\x80\x01\x01\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_UPDATEFLAG = _descriptor.EnumDescriptor(
  name='UpdateFlag',
  full_name='MsgStockDz.UpdateFlag',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='now', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='open', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='high', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='low', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='avg', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='settle', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='hold', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='volume', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='amount', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='last_close', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='last_settle', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='last_hold', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='cur_trade_count', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='trade_count', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='cur_volume', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='rise_limit', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='fall_limit', index=16, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1450,
  serialized_end=1689,
)
_sym_db.RegisterEnumDescriptor(_UPDATEFLAG)

UpdateFlag = enum_type_wrapper.EnumTypeWrapper(_UPDATEFLAG)
_REQTYPENO = _descriptor.EnumDescriptor(
  name='ReqTypeNo',
  full_name='MsgStockDz.ReqTypeNo',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ReqTypeNo_NULL', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Heart_TypeNo', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Market_Push_TypeNo', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Broker_Queue_TypeNo', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Product_Extension_Data_TypeNo', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Market_Expansion_Data_TypeNo', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Other_Data_Extensions_TypeNo', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Data_Update_Notification_TypeNo', index=7, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Product_Extension_Notification_TypeNo', index=8, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Market_Expansion_Notification_TypeNo', index=9, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Other_Data_Extensions_Notification_TypeNo', index=10, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Login_Agreement_TypeNo', index=11, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Error_Packet_TypeNo', index=12, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Kicked_TypeNo', index=13, number=22,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1692,
  serialized_end=2134,
)
_sym_db.RegisterEnumDescriptor(_REQTYPENO)

ReqTypeNo = enum_type_wrapper.EnumTypeWrapper(_REQTYPENO)
now = 0
open = 1
high = 2
low = 3
avg = 4
settle = 5
hold = 6
volume = 7
amount = 8
last_close = 9
last_settle = 10
last_hold = 11
cur_trade_count = 12
trade_count = 13
cur_volume = 14
rise_limit = 15
fall_limit = 16
ReqTypeNo_NULL = 0
Heart_TypeNo = 1
Market_Push_TypeNo = 2
Broker_Queue_TypeNo = 3
Product_Extension_Data_TypeNo = 4
Market_Expansion_Data_TypeNo = 5
Other_Data_Extensions_TypeNo = 6
Data_Update_Notification_TypeNo = 10
Product_Extension_Notification_TypeNo = 11
Market_Expansion_Notification_TypeNo = 12
Other_Data_Extensions_Notification_TypeNo = 13
Login_Agreement_TypeNo = 20
Error_Packet_TypeNo = 21
Kicked_TypeNo = 22



_HEAD = _descriptor.Descriptor(
  name='Head',
  full_name='MsgStockDz.Head',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.Head.type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='packid', full_name='MsgStockDz.Head.packid', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MsgStockDz.Head.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=67,
  serialized_end=117,
)


_LOGIN = _descriptor.Descriptor(
  name='Login',
  full_name='MsgStockDz.Login',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='MsgStockDz.Login.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=141,
)


_ERRORMSG = _descriptor.Descriptor(
  name='ErrorMsg',
  full_name='MsgStockDz.ErrorMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error', full_name='MsgStockDz.ErrorMsg.error', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='MsgStockDz.ErrorMsg.msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=143,
  serialized_end=181,
)


_KICKMSG = _descriptor.Descriptor(
  name='KickMsg',
  full_name='MsgStockDz.KickMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='reason', full_name='MsgStockDz.KickMsg.reason', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='MsgStockDz.KickMsg.msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=183,
  serialized_end=221,
)


_BID = _descriptor.Descriptor(
  name='Bid',
  full_name='MsgStockDz.Bid',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='level', full_name='MsgStockDz.Bid.level', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='price', full_name='MsgStockDz.Bid.price', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='volume', full_name='MsgStockDz.Bid.volume', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='broker_count', full_name='MsgStockDz.Bid.broker_count', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=296,
)


_QUOTE = _descriptor.Descriptor(
  name='Quote',
  full_name='MsgStockDz.Quote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='update_flag', full_name='MsgStockDz.Quote.update_flag', index=0,
      number=1, type=4, cpp_type=4, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStockDz.Quote.market', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStockDz.Quote.code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStockDz.Quote.time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_time', full_name='MsgStockDz.Quote.tick_time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pack_type', full_name='MsgStockDz.Quote.pack_type', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quote_type', full_name='MsgStockDz.Quote.quote_type', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_flag', full_name='MsgStockDz.Quote.tick_flag', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tick_type', full_name='MsgStockDz.Quote.tick_type', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='now', full_name='MsgStockDz.Quote.now', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='open', full_name='MsgStockDz.Quote.open', index=10,
      number=11, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='high', full_name='MsgStockDz.Quote.high', index=11,
      number=12, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='low', full_name='MsgStockDz.Quote.low', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avg', full_name='MsgStockDz.Quote.avg', index=13,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='settle', full_name='MsgStockDz.Quote.settle', index=14,
      number=15, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hold', full_name='MsgStockDz.Quote.hold', index=15,
      number=16, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='volume', full_name='MsgStockDz.Quote.volume', index=16,
      number=17, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='MsgStockDz.Quote.amount', index=17,
      number=18, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_close', full_name='MsgStockDz.Quote.last_close', index=18,
      number=19, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_settle', full_name='MsgStockDz.Quote.last_settle', index=19,
      number=20, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_hold', full_name='MsgStockDz.Quote.last_hold', index=20,
      number=21, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cur_trade_count', full_name='MsgStockDz.Quote.cur_trade_count', index=21,
      number=22, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trade_count', full_name='MsgStockDz.Quote.trade_count', index=22,
      number=23, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cur_volume', full_name='MsgStockDz.Quote.cur_volume', index=23,
      number=24, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rise_limit', full_name='MsgStockDz.Quote.rise_limit', index=24,
      number=25, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fall_limit', full_name='MsgStockDz.Quote.fall_limit', index=25,
      number=26, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sell', full_name='MsgStockDz.Quote.sell', index=26,
      number=27, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='buy', full_name='MsgStockDz.Quote.buy', index=27,
      number=28, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=299,
  serialized_end=879,
)


_BROKER = _descriptor.Descriptor(
  name='Broker',
  full_name='MsgStockDz.Broker',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='broker_level', full_name='MsgStockDz.Broker.broker_level', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='brokerid', full_name='MsgStockDz.Broker.brokerid', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=881,
  serialized_end=929,
)


_BROKERLIST = _descriptor.Descriptor(
  name='BrokerList',
  full_name='MsgStockDz.BrokerList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStockDz.BrokerList.market', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStockDz.BrokerList.code', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.BrokerList.type', index=2,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStockDz.BrokerList.time', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sell', full_name='MsgStockDz.BrokerList.sell', index=4,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='buy', full_name='MsgStockDz.BrokerList.buy', index=5,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=932,
  serialized_end=1097,
)


_SYMEXT = _descriptor.Descriptor(
  name='SymExt',
  full_name='MsgStockDz.SymExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.SymExt.type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStockDz.SymExt.market', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStockDz.SymExt.code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStockDz.SymExt.time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MsgStockDz.SymExt.data', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1099,
  serialized_end=1207,
)


_MARKETEXT = _descriptor.Descriptor(
  name='MarketExt',
  full_name='MsgStockDz.MarketExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.MarketExt.type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStockDz.MarketExt.market', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStockDz.MarketExt.time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MsgStockDz.MarketExt.data', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1209,
  serialized_end=1306,
)


_OTHEREXT = _descriptor.Descriptor(
  name='OtherExt',
  full_name='MsgStockDz.OtherExt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.OtherExt.type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='MsgStockDz.OtherExt.time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='MsgStockDz.OtherExt.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1308,
  serialized_end=1388,
)


_UPDATEEVENT = _descriptor.Descriptor(
  name='UpdateEvent',
  full_name='MsgStockDz.UpdateEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='MsgStockDz.UpdateEvent.type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='market', full_name='MsgStockDz.UpdateEvent.market', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='MsgStockDz.UpdateEvent.code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1390,
  serialized_end=1447,
)

_QUOTE.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_QUOTE.fields_by_name['tick_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_QUOTE.fields_by_name['sell'].message_type = _BID
_QUOTE.fields_by_name['buy'].message_type = _BID
_BROKERLIST.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_BROKERLIST.fields_by_name['sell'].message_type = _BROKER
_BROKERLIST.fields_by_name['buy'].message_type = _BROKER
_SYMEXT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_MARKETEXT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_OTHEREXT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['Head'] = _HEAD
DESCRIPTOR.message_types_by_name['Login'] = _LOGIN
DESCRIPTOR.message_types_by_name['ErrorMsg'] = _ERRORMSG
DESCRIPTOR.message_types_by_name['KickMsg'] = _KICKMSG
DESCRIPTOR.message_types_by_name['Bid'] = _BID
DESCRIPTOR.message_types_by_name['Quote'] = _QUOTE
DESCRIPTOR.message_types_by_name['Broker'] = _BROKER
DESCRIPTOR.message_types_by_name['BrokerList'] = _BROKERLIST
DESCRIPTOR.message_types_by_name['SymExt'] = _SYMEXT
DESCRIPTOR.message_types_by_name['MarketExt'] = _MARKETEXT
DESCRIPTOR.message_types_by_name['OtherExt'] = _OTHEREXT
DESCRIPTOR.message_types_by_name['UpdateEvent'] = _UPDATEEVENT
DESCRIPTOR.enum_types_by_name['UpdateFlag'] = _UPDATEFLAG
DESCRIPTOR.enum_types_by_name['ReqTypeNo'] = _REQTYPENO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Head = _reflection.GeneratedProtocolMessageType('Head', (_message.Message,), {
  'DESCRIPTOR' : _HEAD,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.Head)
  })
_sym_db.RegisterMessage(Head)

Login = _reflection.GeneratedProtocolMessageType('Login', (_message.Message,), {
  'DESCRIPTOR' : _LOGIN,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.Login)
  })
_sym_db.RegisterMessage(Login)

ErrorMsg = _reflection.GeneratedProtocolMessageType('ErrorMsg', (_message.Message,), {
  'DESCRIPTOR' : _ERRORMSG,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.ErrorMsg)
  })
_sym_db.RegisterMessage(ErrorMsg)

KickMsg = _reflection.GeneratedProtocolMessageType('KickMsg', (_message.Message,), {
  'DESCRIPTOR' : _KICKMSG,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.KickMsg)
  })
_sym_db.RegisterMessage(KickMsg)

Bid = _reflection.GeneratedProtocolMessageType('Bid', (_message.Message,), {
  'DESCRIPTOR' : _BID,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.Bid)
  })
_sym_db.RegisterMessage(Bid)

Quote = _reflection.GeneratedProtocolMessageType('Quote', (_message.Message,), {
  'DESCRIPTOR' : _QUOTE,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.Quote)
  })
_sym_db.RegisterMessage(Quote)

Broker = _reflection.GeneratedProtocolMessageType('Broker', (_message.Message,), {
  'DESCRIPTOR' : _BROKER,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.Broker)
  })
_sym_db.RegisterMessage(Broker)

BrokerList = _reflection.GeneratedProtocolMessageType('BrokerList', (_message.Message,), {
  'DESCRIPTOR' : _BROKERLIST,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.BrokerList)
  })
_sym_db.RegisterMessage(BrokerList)

SymExt = _reflection.GeneratedProtocolMessageType('SymExt', (_message.Message,), {
  'DESCRIPTOR' : _SYMEXT,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.SymExt)
  })
_sym_db.RegisterMessage(SymExt)

MarketExt = _reflection.GeneratedProtocolMessageType('MarketExt', (_message.Message,), {
  'DESCRIPTOR' : _MARKETEXT,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.MarketExt)
  })
_sym_db.RegisterMessage(MarketExt)

OtherExt = _reflection.GeneratedProtocolMessageType('OtherExt', (_message.Message,), {
  'DESCRIPTOR' : _OTHEREXT,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.OtherExt)
  })
_sym_db.RegisterMessage(OtherExt)

UpdateEvent = _reflection.GeneratedProtocolMessageType('UpdateEvent', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEVENT,
  '__module__' : 'msg_stock_dz_pb2'
  # @@protoc_insertion_point(class_scope:MsgStockDz.UpdateEvent)
  })
_sym_db.RegisterMessage(UpdateEvent)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)




# -*- coding: utf-8 -*-

# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# KaisaGlobal rights are reserved.
# ===================================================================

"""
This code provide basic encryption function for KaisaGlobal-Open.

developed by KaisaGlobal quant team.
2020.12.11
"""


import json
import requests
import base64
import random
from urllib import parse

import Crypto
from Crypto.Cipher import PKCS1_v1_5
from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Cipher import AES
from hashlib import md5


class KaisaCrypto(object):

    def __init__(self):

        self.aes_secret_key_secret = ""

    def encrypt_aes_password(self, text, type="MARKET", AES_SECRET_KEY=None):

        IV = "kai&sa!global@)!"
        if AES_SECRET_KEY is None:
            if type=="TRADE":
                AES_SECRET_KEY = 'login&pwd@glob)!'
            elif type=="MARKET":
                AES_SECRET_KEY = "third&log@glob)!"
            elif type=="SECRET":
                AES_SECRET_KEY = self.aes_secret_key_secret
            else:
                AES_SECRET_KEY = "third&log@glob)!"
        # padding algo.
        BS = len(AES_SECRET_KEY)
        pad = lambda s: s + (BS - len(s) % BS) * chr(BS - len(s) % BS)
        unpad = lambda s: s[0:-ord(s[-1:])]
        cryptor = AES.new(AES_SECRET_KEY.encode("utf8"), AES.MODE_CBC, IV.encode("utf8"))
        ciphertext = cryptor.encrypt(bytes(pad(text), encoding="utf8"))
        encrypt_password = base64.b64encode(ciphertext)
        password = parse.quote(encrypt_password, "\\")
        return password

    def encrypt_aes_password_forQ(self, text, type="MARKET", AES_SECRET_KEY=None):

        IV = "kai&sa!global@)!"
        if AES_SECRET_KEY is None:
            if type == "TRADE":
                AES_SECRET_KEY = 'login&pwd@glob)!'
            elif type == "MARKET":
                AES_SECRET_KEY = "third&log@glob)!"
            elif type == "SECRET":
                AES_SECRET_KEY = self.aes_secret_key_secret
            else:
                AES_SECRET_KEY = "third&log@glob)!"
        # padding algo.
        BS = len(AES_SECRET_KEY)
        PADDING = '\n'
        pad = lambda s: s + (BS - len(s) % BS) * chr(BS - len(s) % BS)
        cryptor = AES.new(AES_SECRET_KEY.encode("utf8"), AES.MODE_CBC, IV.encode("utf8"))
        ciphertext = cryptor.encrypt(bytes(pad(text), encoding="utf8"))
        encrypt_password = base64.b64encode(ciphertext)
        encrypt_password = encrypt_password.decode("utf8")
        return encrypt_password

    def decrypt_aes_password(self, encrypted, AES_SECRET_KEY):

        # encrypt_password
        IV = "kai&sa!global@)!"
        decrypted = base64.b64decode(encrypted)
        cryptor = AES.new(AES_SECRET_KEY.encode("utf8"), AES.MODE_CBC, IV.encode("utf8"))
        decrypted = cryptor.decrypt(decrypted)
        unpad = lambda s: s[0:-ord(s[-1:])]
        decrypted = unpad(decrypted)
        return decrypted

    def encrypt_rsa_username(self, username, crypto_rsa_url):

        account_rsa = requests.get(url=crypto_rsa_url, params={'username': username})
        if (account_rsa.status_code//100!=2):
            return None
        pubkey = json.loads(account_rsa.text)["body"]
        pubkey_lines = []
        for i in range(len(pubkey )//64 +1):
            str_ = pubkey[(i) *64:( i+1 ) *64]
            if len(str_ ) >0:
                pubkey_lines.append(str_)
        pubkey_lines = ["-----BEGIN RSA PUBLIC KEY-----"] + pubkey_lines + ["-----END RSA PUBLIC KEY-----"]
        pubkey = "\n".join(pubkey_lines)

        cipher_public = PKCS1_v1_5.new(RSA.importKey(pubkey.encode()))
        cipher_text = base64.b64encode(cipher_public.encrypt(username.encode()))
        username_rsa = cipher_text.decode()
        return username_rsa


    def encrypt_rsa_secretQ(self, crypto_rsa_url):

        private_text = self._produce_encrypt_privatekey()
        account_rsa = requests.get(url=crypto_rsa_url)
        if (account_rsa.status_code//100!=2):
            return None
        pubkey = json.loads(account_rsa.text)["body"]
        pubkey_lines = []
        for i in range(len(pubkey)//64 +1):
            str_ = pubkey[(i) *64:( i+1 ) *64]
            if len(str_ ) >0:
                pubkey_lines.append(str_)
        pubkey_lines = ["-----BEGIN RSA PUBLIC KEY-----"] + pubkey_lines + ["-----END RSA PUBLIC KEY-----"]
        pubkey = "\n".join(pubkey_lines)
        cipher_public = PKCS1_v1_5.new(RSA.importKey(pubkey.encode()))
        cipher_text = base64.b64encode(cipher_public.encrypt(private_text.encode()))
        cipher_text_rsa = cipher_text.decode()
        return cipher_text_rsa

    def encrypt_md5(self, str_):
        obj_md5 = md5()
        obj_md5.update(str_.encode(encoding='utf-8'))
        return obj_md5.hexdigest()

    def _produce_encrypt_privatekey(self, length=16):

        H = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
        str_ = ""
        for i in range(length):
            str_+= random.choice(H)
        self.aes_secret_key_secret = str_
        return str_

    def get_encrypt_private(self):
        return self.private_key_trades





# -*- coding: utf-8 -*-

# ===================================================================
# This example is written by Kaisa FinTech Quant Team.
# KaisaGlobal OpenAPI provides a friendly way to trade in Global Markets.
# and now Hong Kong market is suported.
# This code is shown to our clients how to query information of account.
# Dec. 14, 2020
# ===================================================================

import os, sys, json
import datetime, time
import numpy as np
import pandas as pd
from copy import deepcopy
from kaisaglobalapi.common import KaisaGateway

my_token = "__my_token__"

class KaisaDemo(KaisaGateway):
    def __init__(self, *args, **kwargs):
        super(KaisaGateway, self).__init__(*args, **kwargs)

    def on_market_data(self, data):
        print("\n**** market ws receive ****")
        print(data)

    def on_trade_data(self, data):
        print("\n**** trade ws receice ****")
        print(data)


if __name__=="__main__":

    default_config = {
        "user_id": "86,182******",
        "user_pwd": "******",
        "account_code": "9604******",
        "account_pwd": "******",
        "environment": "actual",
        "openapi_token": my_token,
        "ip_address": "104.193.**.**"
    }

    KaisaGw = KaisaDemo()
    KaisaGw.jq_user_config(**default_config)
    auth_status, auth_msg = KaisaGw.gateway_auth()
    if auth_status:
        print("gateway connect success.")
    else:
        print("gateway connect fail.")

    ### connect to trade.
    ret_ok, ret_msg = KaisaGw.trade_connect()
    if ret_ok:
        print('trade login success.')
    else:
        print('trade login fail.')
        print(ret_msg)

    ### trade auth.
    ret_ok, ret_msg = KaisaGw.trade_auth()
    if ret_ok:
        print("trade auth success.")
    else:
        print("trade auth fail. as follows.")
        print(ret_msg)

    ### query porfolio of client's account
    ret_ok, ret_data = KaisaGw.query_portfolio()
    if ret_ok:
        print('query portfolio success.')
        print(ret_data)
    else:
        print("query portfolio fail.")
        print(ret_data)

    ### query position of client's account
    ret_ok, ret_data = KaisaGw.query_position()
    if ret_ok:
        print("query position success.")
        print(ret_data)
    else:
        print("query position success.")
        print(ret_data)

    ### query order of client's account
    ret_ok, ret_data = KaisaGw.query_order()
    if ret_ok:
        print("query order success.")
        print(ret_data)
    else:
        print("query order fail.")
        print(ret_data)

